/*  fio.h
    Abbreviated version of fileio.h from the prosuite sources.
    This version is for use with my MANX library fileio32.lib.
    This lib allows me to use the file requestor with out having
    all the source with every program that uses it.
       Joel Swank 9/20/89                                                  */

/* === General FileIO Declarations ======================================= */
#define MAX_NAME_LENGTH 		64		/* Includes terminating NULL */
#define MAX_TOOLTYPE_LENGTH	80		/* Includes terminating NULL */
#define VISIBLE_SELECT_LENGTH	16		/* Buffer for visible chars + '\0' */

/* The WBENCH_CODE definition is used to define whether to include 
 * or not include the Workbench-style .info file pattern matching code.
 * If you intend on using Workbench pattern matching, or if you are 
 * going to give the user the option, define this constant.
 * This makes sure that the Workbench-style code is in included in the 
 * FileIO files when they are compiled.
 * To attempt to match Workbench patterns at run time, you must set 
 * the WBENCH_MATCH flag in your FileIOSupport structure.
 * 
 * If you aren't interested in Workbench-style pattern matching 
 * and are interested in saving about 500 bytes of code size, 
 * comment out the WBENCH_CODE definition.
 */
#define WBENCH_CODE





/* === FileIO Support Structure ========================================== */
struct FileIOSupport
	{
	SHORT Flags;

	UBYTE *ReqTitle;	/* The text that will be the requester's title */

	/* After a successful call to GetFileIOName(), these fields will have
	 * the names selected by the user.  You should never have to initialize
	 * these fields, only read from them, though initializing them won't hurt.
	 */
	UBYTE FileName[MAX_NAME_LENGTH];
	UBYTE DrawerName[MAX_NAME_LENGTH];
	UBYTE DiskName[MAX_NAME_LENGTH];

	/* If the LOCK_GOTTEN flag is set, the lock can be found here. */
	ULONG DOSLock;

	SHORT NameCount;
	SHORT NameStart;
	SHORT CurrentPick;
	struct Remember *NameKey;

	SHORT VolumeIndex;
	SHORT VolumeCount;
	struct Remember *VolumeKey;

	SHORT DiskObjectType;
	UBYTE ToolType[MAX_TOOLTYPE_LENGTH];

	LONG UserData;		/* Use this for anything you want */
	};

